==> fbat_1.0.0.tar.gz: fbat/R/FBAT.R <==
# fbat analysis

# geneSetObj -- a geneSet object
# model -- genotype coding method. 
#      "d" -- GDOM (dominante);
#      "r" -- GREC (recessive); 
#      "g" -- GEN (genotype);
#      "a" or otherwise -- GTDT (additive)
# traitMethod -- trait coding method. 
#      traitMethod=1 -- y-offset; 
#      traitMethod=2 -- T=1 if affected, T=0 otherwise
# traitOffset -- offset if traitMethod=1.
# quiet -- print intermediate results if quiet=FALSE
fbat<-function(geneSetObj, model="a", traitMethod=3, traitOffset=0, quiet=TRUE)
{
  cat("converting geneSet object to numerical matrix...\n")
  tmp<-geneSet2Ped(geneSetObj)
  ped<-tmp$ped
  columns<-tmp$columns

  res<-fbat.default(ped, columns, model, traitMethod, traitOffset, quiet)
  invisible(res)
}

# ped -- pedigrees
# model -- genotype coding method. 
#      "d" -- GDOM (dominante);
#      "r" -- GREC (recessive); 
#      "g" -- GEN (genotype);
#      "a" or otherwise -- GTDT (additive)
# traitMethod -- trait coding method. 
#      traitMethod=1 -- y-offset; 
#      traitMethod=2 -- T=1 if affected, T=0 otherwise
# traitOffset -- offset if traitMethod=1.
# quiet -- print intermediate results if quiet=FALSE
fbat.default<-function(ped, 
                       columns = c("family", "pid", "father", 
                                   "mother", "sex", "affected"),
                       model="a", 
                       traitMethod=3, 
                       traitOffset=0, 
                       quiet=TRUE)
{
  tmp<-checkColumn(ped, columns)
  ped<-tmp$ped
  columns<-tmp$columns

  if(quiet==FALSE)
  { cat("checking alleles ...\n") }
  # check if number of alleles of a marker > 2
  tmp<-checkAlleles(ped, columns)
  alleles<-tmp$alleles
  nAllele<-tmp$nAllele
  nMarker<-length(nAllele)
  family<-as.numeric(ped[,1])
  family.u<-unique(family) # unique family id
  nFamily<-length(family.u) # number of family
  familySize<-tapply(rep(1,length(family)), family, sum) # number of members of families
  markerNames<-getMarkerNames(ped, columns)

  if(quiet==FALSE)
  { cat("checking compatibility ...\n") }
  # check compatibility
  sampleInfoLen<-length(columns)
  errorFlag<-checkCompatibility(ped, alleles, nAllele, sampleInfoLen, nFamily, familySize, markerNames, quiet)
  if(errorFlag!=0)
  { stop("Error occurs!\n"); }


  nr<-nrow(ped) # total number of individuals in the pedgree file
  nc<-ncol(ped) # 6+2*nMarker, where nMarker is the number of markers


  genoMethod<-match(model, c("d","r","g","a"),nomatch=4)

  # length of coded genotype for each marker
  nXvec<-tapply(nAllele, 1:nMarker, lengthX, genoMethod=genoMethod)

  # length of scores
  nS<-sum(nXvec)
  nS2<-sum(nXvec^2)

  # sort ped according to pedigree id (i.e. family)
  ped2<-ped[order(ped[,1]),]

  # in R, a matrix is recorded column by column. In C, a matrix is 
  # recorded row by row. So to use R to call C function, we have to
  # first transpose 'ped' first.
  ped2<-as.integer(as.vector(t(ped2)))
  nr<-as.integer(nr)
  nc<-as.integer(nc)
  nFamily<-as.integer(nFamily)
  familySize<-as.integer(familySize)
  genoMethod<-as.integer(genoMethod)
  traitMethod<-as.integer(traitMethod)
  traitOffset<-as.double(traitOffset)
  alleles<-as.integer(alleles)
  nAllele<-as.integer(nAllele)

  cat("fbating...\n")
  # call C function fbat()
  res<-.C("fbat", ped2, nr, nc, nFamily,
          familySize, genoMethod, traitMethod,
          traitOffset, alleles, nAllele,
          S=as.double(rep(0,nS)),
          ES=as.double(rep(0,nS)), 
          CovS=as.double(rep(0,nS2)),
          numInfoFamily=as.integer(rep(0, nMarker)),
          errorFlag=as.integer(0),
          quiet=as.integer(quiet),
          PACKAGE="fbat"
          )


  # get vector form of results of fbat analysis
  S<-as.double(res$S)
  ES<-as.double(res$ES)
  CovS<-as.double(res$CovS)

  # number of informative family for each marker
  numInfoFamily<-as.integer(res$numInfoFamily)
  names(numInfoFamily)<-markerNames


  # get list format results

  names.S.list<-paste("S.", markerNames, sep="")
  names.ES.list<-paste("ES.", markerNames, sep="")
  names.CovS.list<-paste("CovS.", markerNames, sep="")

  S.list<-list(S[1:nXvec[1]])
  ES.list<-list(ES[1:nXvec[1]])
  CovS.list<-list( matrix(CovS[ 1:(nXvec[1]^2) ],nrow=nXvec[1], 
    ncol=nXvec[1]) )

  if(nMarker>1)
  {
    for(M in 2:nMarker)
    { 
      mystart<-sum(nXvec[1:(M-1)])+1
      myend<-sum(nXvec[1:M])
      S.list[[M]]<-S[mystart:myend]
      ES.list[[M]]<-ES[mystart:myend]
      mystart<-sum( (nXvec[1:(M-1)])^2 ) + 1
      myend<-sum( (nXvec[1:M])^2 )
      CovS.list[[M]]<-matrix(CovS[mystart:myend], nrow=nXvec[M], ncol=nXvec[M])
    }
  }
  names(S.list)<-names.S.list

  names(ES.list)<-names.ES.list
  names(CovS.list)<-names.CovS.list


  # get fbat statistic and pvalue for each marker.
  # First column is fbat statistic; 
  # Second column is the degree of freedom
  # Third column is corresponding pvalue
  statPvalue<-matrix(0,nrow=nMarker, ncol=3)
  colnames(statPvalue)<-c("chisq","rank", "pvalue")
  rownames(statPvalue)<-markerNames

  # for each marker
  for(M in 1:nMarker)
  { SM<-S.list[[M]] 
    ESM<-ES.list[[M]]
    CovSM<-CovS.list[[M]]
    tmp<-fbatStatM(SM, ESM, CovSM) 
    statPvalue[M,1]<-tmp$stat
    statPvalue[M,2]<-tmp$r
    statPvalue[M,3]<-tmp$pvalue
  }

  if(quiet==FALSE)
  { cat("********** Test statistics and p-values ***********\n")
    print(statPvalue) 
  }

  alleles.list<-list(alleles[1:nAllele[1]])
  if(nMarker>1)
  {
    for(M in 2:nMarker)
    {
      mystart<-sum(nAllele[1:(M-1)])+1
      myend<-sum(nAllele[1:M])
      alleles.list[[M]]<-alleles[mystart:myend]
    }
  }
  names(alleles.list)<-markerNames

  names(familySize)<-family.u

  invisible(list(statPvalue=statPvalue, S.list=S.list, ES.list=ES.list, 
    CovS.list=CovS.list, alleles.list=alleles.list, familySize=familySize, 
    numInfoFamily=numInfoFamily))
}


# obtain test statistic for a marker
# SM -- 1 by 2 vector of score for marker M
# ESM -- 1 by 2 vector of expected score for marker M
# CovSM -- 2 by 2 covariance matrix of SM
fbatStatM<-function(SM, ESM, CovSM)
{
  UM<-as.vector(SM-ESM)

  # degree of freedom is equal to rank of CovSM
  r<-matRank(CovSM)
  if(r>0)
  { # test statistic
    stat<-t(UM)%*%ginv(CovSM)%*%UM

    # asymptotic pvalue
    pvalue<-1-pchisq(stat, df=r)
  } else {
    stat<-0
    pvalue<-0
  }

  invisible(list(stat=stat, r=r, pvalue=pvalue))
}

# rank of matrix A
matRank<-function(A, tol=sqrt(.Machine$double.eps))
{
  # singular value decomposition
  d<-svd(A)$d # singular values
  # rank(A)= number of non-zero singular values
  pos<-which(abs(d)>tol)
  invisible(length(pos))
}

# obtain the length of coded genotype for a markers
lengthX<-function(nAlleleM, genoMethod)
{
  if((genoMethod)!=3) # not GEO method
  { nX=nAlleleM; }
  else {
    nX=nAlleleM*(nAlleleM+1)/2;
  }
  invisible(nX)
}


## get markernames
getMarkerNames <- function(ped, columns=c("family", "pid", "father", "mother", "sex", "affected"))
{

  len <- length(columns)
  pedColNames <- colnames(ped)
  pedColNames2 <- pedColNames[-(1:len)]
  tmplabels <- unlist(strsplit(pedColNames2, "\\."))
  tmplen <- length(tmplabels)
  markerNames <- tmplabels[seq(from=1, to=tmplen, by=4)]

  return(markerNames)
}

==> fbat_1.10.0.tar.gz: fbat/R/FBAT.R <==
# fbat analysis

# geneSetObj -- a geneSet object
# model -- genotype coding method. 
#      "d" -- GDOM (dominante);
#      "r" -- GREC (recessive); 
#      "g" -- GEN (genotype);
#      "a" or otherwise -- GTDT (additive)
# traitMethod -- trait coding method. 
#      traitMethod=1 -- y-offset; 
#      traitMethod=2 -- T=1 if affected, T=0 otherwise
# traitOffset -- offset if traitMethod=1.
# quiet -- print intermediate results if quiet=FALSE
fbat<-function(geneSetObj, model="a", traitMethod=3, traitOffset=0, quiet=TRUE)
{
  if(!quiet)
    cat("converting geneSet object to numerical matrix...\n")

  pedObj<-geneSet2Ped(geneSetObj)

  if(!quiet)
    cat("start fbat ...\n")

  res<-fbat.default(pedObj, model, traitMethod, traitOffset, quiet)
  invisible(res)
}

# ped -- pedigrees
# model -- genotype coding method. 
#      "d" -- GDOM (dominante);
#      "r" -- GREC (recessive); 
#      "g" -- GEN (genotype);
#      "a" or otherwise -- GTDT (additive)
# traitMethod -- trait coding method. 
#      traitMethod=1 -- y-offset; 
#      traitMethod=2 -- T=1 if affected, T=0 otherwise
# traitOffset -- offset if traitMethod=1.
# quiet -- print intermediate results if quiet=FALSE
fbat.default<-function(pedObj, 
                       model="a", 
                       traitMethod=3, 
                       traitOffset=0, 
                       quiet=TRUE)
{
  pedObj <- checkColumn(pedObj)
  ped<-pedObj$ped
  columns<-pedObj$columns
  markerNames<-pedObj$markerNames
  nMarkers<-length(markerNames)

  if(!quiet)
    cat("checking alleles ...\n") 

  # check if number of alleles of a marker > 2
  tmp<-getAlleles(pedObj)
  alleles<-tmp$alleles
  nAllele<-tmp$nAllele

  ## check if number of alleles for a marker is greater than 2
  pos <- which(nAllele>2)
  if(length(pos)>0)
    {
      stop("Markers ", paste("`", markerNames[pos], "'", sep=", "), " ",
           "have more than 2 alleles.  Only bi-allelic markers ", 
           "are currently supported.")
    }

  family<-as.numeric(ped[,1])
  family.u<-unique(family) # unique family id
  nFamily<-length(family.u) # number of family
  familySize<-tapply(rep(1,length(family)), family, sum) # number of members of families

  if(!quiet)
    cat("checking compatibility ...\n")

  # check compatibility
  sampleInfoLen<-length(columns)
  tmp<-checkCompatibility(ped, alleles, nAllele, sampleInfoLen, nFamily, 
                          familySize, markerNames, quiet)
  errorFlag<-tmp$errorFlag
  compatibleFlag<-tmp$compatibleFlag

  # number of Mendelian error by markers
  nMerrMarker<-tmp$nMerrMarker
  # number of Mendelian error by families
  nMerrFamily<-tmp$nMerrFamily
  # number of errors for sample ino by families
  nErrFamilySample<-tmp$nErrFamilySample

  if(errorFlag!=0)
  { stop("Error occurs!\n"); }


  nr<-nrow(ped) # total number of individuals in the pedgree file
  nc<-ncol(ped) # 6+2*nMarkers, where nMarkers is the number of markers


  genoMethod<-match(model, c("d","r","g","a"),nomatch=4)

  # length of coded genotype for each marker
  nXvec<-tapply(nAllele, 1:nMarkers, lengthX, genoMethod=genoMethod)

  # length of scores
  nS<-sum(nXvec)
  nS2<-sum(nXvec^2)

  # sort ped according to pedigree id (i.e. family)
  ped2<-ped[order(ped[,1]),]

  # in R, a matrix is recorded column by column. In C, a matrix is 
  # recorded row by row. So to use R to call C function, we have to
  # first transpose 'ped' first.
  ped2<-as.integer(as.vector(t(ped2)))
  nr<-as.integer(nr)
  nc<-as.integer(nc)
  nFamily<-as.integer(nFamily)
  familySize<-as.integer(familySize)
  genoMethod<-as.integer(genoMethod)
  traitMethod<-as.integer(traitMethod)
  traitOffset<-as.double(traitOffset)
  alleles<-as.integer(alleles)
  nAllele<-as.integer(nAllele)

  # call C function fbat()
  res<-.C("fbat", ped2, nr, nc, nFamily,
          familySize, genoMethod, traitMethod,
          traitOffset, alleles, nAllele,
          S=as.double(rep(0,nS)),
          ES=as.double(rep(0,nS)), 
          CovS=as.double(rep(0,nS2)),
          numInfoFamily=as.integer(rep(0, nMarkers)),
          errorFlag=as.integer(0),
          quiet=as.integer(quiet),
          PACKAGE="fbat"
          )


  # get vector form of results of fbat analysis
  S<-as.double(res$S)
  ES<-as.double(res$ES)
  CovS<-as.double(res$CovS)

  # number of informative family for each marker
  numInfoFamily<-as.integer(res$numInfoFamily)
  names(numInfoFamily)<-markerNames


  # get list format results

  names.S.list<-paste("S.", markerNames, sep="")
  names.ES.list<-paste("ES.", markerNames, sep="")
  names.CovS.list<-paste("CovS.", markerNames, sep="")

  S.list<-list(S[1:nXvec[1]])
  ES.list<-list(ES[1:nXvec[1]])
  CovS.list<-list( matrix(CovS[ 1:(nXvec[1]^2) ],nrow=nXvec[1], 
    ncol=nXvec[1]) )

  if(nMarkers>1)
  {
    for(M in 2:nMarkers)
    { 
      mystart<-sum(nXvec[1:(M-1)])+1
      myend<-sum(nXvec[1:M])
      S.list[[M]]<-S[mystart:myend]
      ES.list[[M]]<-ES[mystart:myend]
      mystart<-sum( (nXvec[1:(M-1)])^2 ) + 1
      myend<-sum( (nXvec[1:M])^2 )
      CovS.list[[M]]<-matrix(CovS[mystart:myend], nrow=nXvec[M], ncol=nXvec[M])
    }
  }
  names(S.list)<-names.S.list

  names(ES.list)<-names.ES.list
  names(CovS.list)<-names.CovS.list


  # get fbat statistic and pvalue for each marker.
  # First column is fbat statistic; 
  # Second column is the degree of freedom
  # Third column is corresponding pvalue
  statPvalue<-matrix(0,nrow=nMarkers, ncol=3)
  colnames(statPvalue)<-c("chisq","rank", "pvalue")
  rownames(statPvalue)<-markerNames

  # for each marker
  for(M in 1:nMarkers)
  { SM<-S.list[[M]] 
    ESM<-ES.list[[M]]
    CovSM<-CovS.list[[M]]
    tmp<-fbatStatM(SM, ESM, CovSM) 
    statPvalue[M,1]<-tmp$stat
    statPvalue[M,2]<-tmp$r
    statPvalue[M,3]<-tmp$pvalue
  }

  if(!quiet)
  { cat("********** Test statistics and p-values ***********\n")
    print(statPvalue) 
  }

  alleles.list<-list(alleles[1:nAllele[1]])
  if(nMarkers>1)
  {
    for(M in 2:nMarkers)
    {
      mystart<-sum(nAllele[1:(M-1)])+1
      myend<-sum(nAllele[1:M])
      alleles.list[[M]]<-alleles[mystart:myend]
    }
  }
  names(alleles.list)<-markerNames

  names(familySize)<-family.u

  invisible(list(statPvalue=statPvalue, S.list=S.list, ES.list=ES.list, 
    CovS.list=CovS.list, alleles.list=alleles.list, familySize=familySize, 
    numInfoFamily=numInfoFamily,
    nMerrMarker=nMerrMarker, nMerrFamily=nMerrFamily,
    nErrFamilySample=nErrFamilySample))
}


# obtain test statistic for a marker
# SM -- 1 by 2 vector of score for marker M
# ESM -- 1 by 2 vector of expected score for marker M
# CovSM -- 2 by 2 covariance matrix of SM
fbatStatM<-function(SM, ESM, CovSM)
{
  UM<-as.vector(SM-ESM)

  # degree of freedom is equal to rank of CovSM
  r<-matRank(CovSM)
  if(r>0)
  { # test statistic
    stat<-t(UM)%*%ginv(CovSM)%*%UM

    # asymptotic pvalue
    pvalue<-1-pchisq(stat, df=r)
  } else {
    stat<-0
    pvalue<-0
  }

  invisible(list(stat=stat, r=r, pvalue=pvalue))
}

# rank of matrix A
matRank<-function(A, tol=sqrt(.Machine$double.eps))
{
  # singular value decomposition
  d<-svd(A)$d # singular values
  # rank(A)= number of non-zero singular values
  pos<-which(abs(d)>tol)
  invisible(length(pos))
}

# obtain the length of coded genotype for a markers
lengthX<-function(nAlleleM, genoMethod)
{
  if((genoMethod)!=3) # not GEO method
  { nX=nAlleleM; }
  else {
    nX=nAlleleM*(nAlleleM+1)/2;
  }
  invisible(nX)
}


## get markernames
getMarkerNames <- function(ped, columns=c("family", "pid", "father", "mother", "sex", "affected"))
{

  len <- length(columns)
  pedColNames <- colnames(ped)
  pedColNames2 <- pedColNames[-(1:len)]
  tmplabels <- unlist(strsplit(pedColNames2, "\\."))
  tmplen <- length(tmplabels)
  markerNames <- tmplabels[seq(from=1, to=tmplen, by=4)]

  return(markerNames)
}
